@@ -5,16 +5,12 @@
 
 import mimetypes
 import json
-
-try: #PY3
-    from base64 import decodebytes
-except ImportError: #PY2
-    from base64 import decodestring as decodebytes
-
+from base64 import decodebytes
 
 from tornado import web
 
 from notebook.base.handlers import IPythonHandler
+from notebook.utils import maybe_future
 
 
 class FilesHandler(IPythonHandler):
@@ -54,13 +50,13 @@ class FilesHandler(IPythonHandler):
         else:
             name = path
         
-        model = cm.get(path, type='file', content=include_body)
+        model = yield maybe_future(cm.get(path, type='file', content=include_body))
         
         if self.get_argument("download", False):
             self.set_attachment_header(name)
         
         # get mimetype from filename
-        if name.endswith('.ipynb'):
+        if name.lower().endswith('.ipynb'):
             self.set_header('Content-Type', 'application/x-ipynb+json')
         else:
             cur_mime = mimetypes.guess_type(name)[0]
