@@ -91,6 +91,22 @@ class NotebookTestBase(TestCase):
             url_path_join(cls.base_url(), path),
             **kwargs)
         return response
+
+    @classmethod
+    def get_patch_env(cls):
+        return {
+            'HOME': cls.home_dir,
+            'PYTHONPATH': os.pathsep.join(sys.path),
+            'IPYTHONDIR': pjoin(cls.home_dir, '.ipython'),
+            'JUPYTER_NO_CONFIG': '1', # needed in the future
+            'JUPYTER_CONFIG_DIR' : cls.config_dir,
+            'JUPYTER_DATA_DIR' : cls.data_dir,
+            'JUPYTER_RUNTIME_DIR': cls.runtime_dir,
+        }
+
+    @classmethod
+    def get_argv(cls):
+        return []
     
     @classmethod
     def setup_class(cls):
@@ -109,15 +125,7 @@ class NotebookTestBase(TestCase):
         config_dir = cls.config_dir = tmp('config')
         runtime_dir = cls.runtime_dir = tmp('runtime')
         cls.notebook_dir = tmp('notebooks')
-        cls.env_patch = patch.dict('os.environ', {
-            'HOME': cls.home_dir,
-            'PYTHONPATH': os.pathsep.join(sys.path),
-            'IPYTHONDIR': pjoin(cls.home_dir, '.ipython'),
-            'JUPYTER_NO_CONFIG': '1', # needed in the future
-            'JUPYTER_CONFIG_DIR' : config_dir,
-            'JUPYTER_DATA_DIR' : data_dir,
-            'JUPYTER_RUNTIME_DIR': runtime_dir,
-        })
+        cls.env_patch = patch.dict('os.environ', cls.get_patch_env())
         cls.env_patch.start()
         cls.path_patch = patch.multiple(
             jupyter_core.paths,
@@ -157,7 +165,7 @@ class NotebookTestBase(TestCase):
             # needs to be redone after initialize, which reconfigures logging
             app.log.propagate = True
             app.log.handlers = []
-            app.initialize(argv=[])
+            app.initialize(argv=cls.get_argv())
             app.log.propagate = True
             app.log.handlers = []
             loop = IOLoop.current()
